"""
@generated by codestare-proto-plus.  Do not edit manually!

Note:
    __init__.py files for packages are autogenerated, and import all messages from submodules to keep import names consistent with the directory structure of the ``.proto`` source files.
"""
from .topicSubscription_pb_plus import (
    TopicSubscription,
)


__all__ = (
    "TopicSubscription",
)


"""
@generated by codestare-proto-plus.  Do not edit manually!
"""
from proto import (
    Field,
    module,
)

from proto.message import (
    Message,
)

from ubii.proto.v1.general.error_pb_plus import (
    Error,
)

from ubii.proto.v1.topicData.topicDataRecord_pb_plus import (
    TopicDataRecord,
    TopicDataRecordList,
)


__protobuf__ = module(
    package="ubii.proto.v1",
    manifest={
        "TopicData",
    }
)


class TopicData(Message):
    """Generated from ubii/proto/v1/topicData/topicData.proto

    Attributes:
        topic_data_record (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~ubii.proto.v1.topicData.topicDataRecord_pb_plus.TopicDataRecord`

        topic_data_record_list (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~ubii.proto.v1.topicData.topicDataRecord_pb_plus.TopicDataRecordList`

        error (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~ubii.proto.v1.general.error_pb_plus.Error`

    """

    topic_data_record = Field(
        TopicDataRecord,
        number=2,
        oneof='type',
    )
    topic_data_record_list = Field(
        TopicDataRecordList,
        number=3,
        oneof='type',
    )
    error = Field(
        Error,
        number=4,
        oneof='type',
    )




"""
@generated by codestare-proto-plus.  Do not edit manually!
"""
from proto import (
    Enum,
    Field,
    RepeatedField,
    STRING,
    module,
)

from proto.message import (
    Message,
)

from ubii.proto.v1.devices.component_pb_plus import (
    Component,
)


__protobuf__ = module(
    package="ubii.proto.v1",
    manifest={
        "Device",
        "DeviceList",
    }
)


class Device(Message):
    """Generated from ubii/proto/v1/devices/device.proto

    Attributes:
        id (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~proto.STRING`

        name (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~proto.STRING`

        device_type (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~.DeviceType`

        components (proto.fields.RepeatedField): :obj:`~proto.fields.RepeatedField` of
            type :obj:`~ubii.proto.v1.devices.component_pb_plus.Component`

        client_id (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~proto.STRING`

        tags (proto.fields.RepeatedField): :obj:`~proto.fields.RepeatedField` of type
            :obj:`~proto.STRING`

        description (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~proto.STRING`

    """

    class DeviceType(Enum):
        PARTICIPANT = 0
        WATCHER = 1

    id = Field(
        STRING,
        number=1,
    )
    name = Field(
        STRING,
        number=2,
    )
    device_type = Field(
        DeviceType,
        number=3,
    )
    components = RepeatedField(
        Component,
        number=4,
    )
    client_id = Field(
        STRING,
        number=5,
    )
    tags = RepeatedField(
        STRING,
        number=6,
    )
    description = Field(
        STRING,
        number=7,
    )


class DeviceList(Message):
    """Generated from ubii/proto/v1/devices/device.proto

    Attributes:
        elements (proto.fields.RepeatedField): :obj:`~proto.fields.RepeatedField` of
            type :obj:`~.Device`

    """

    elements = RepeatedField(
        Device,
        number=1,
    )




"""
@generated by codestare-proto-plus.  Do not edit manually!
"""
from proto import (
    Field,
    module,
)

from proto.message import (
    Message,
)

from ubii.proto.v1.clients.client_pb_plus import (
    Client,
    ClientList,
)

from ubii.proto.v1.dataStructure.lists_pb_plus import (
    StringList,
)

from ubii.proto.v1.devices.device_pb_plus import (
    Device,
    DeviceList,
)

from ubii.proto.v1.devices.topicDemux_pb_plus import (
    TopicDemux,
    TopicDemuxList,
)

from ubii.proto.v1.devices.topicMux_pb_plus import (
    TopicMux,
    TopicMuxList,
)

from ubii.proto.v1.general.error_pb_plus import (
    Error,
)

from ubii.proto.v1.general.success_pb_plus import (
    Success,
)

from ubii.proto.v1.processing.lockstepProcessing_pb_plus import (
    LockstepProcessingReply,
)

from ubii.proto.v1.processing.processingModule_pb_plus import (
    ProcessingModule,
    ProcessingModuleList,
)

from ubii.proto.v1.servers.server_pb_plus import (
    Server,
)

from ubii.proto.v1.services.service_pb_plus import (
    Service,
    ServiceList,
)

from ubii.proto.v1.sessions.session_pb_plus import (
    Session,
    SessionList,
)


__protobuf__ = module(
    package="ubii.proto.v1",
    manifest={
        "ServiceReply",
    }
)


class ServiceReply(Message):
    """Generated from ubii/proto/v1/services/serviceReply.proto

    Attributes:
        success (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~ubii.proto.v1.general.success_pb_plus.Success`

        error (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~ubii.proto.v1.general.error_pb_plus.Error`

        client (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~ubii.proto.v1.clients.client_pb_plus.Client`

        device (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~ubii.proto.v1.devices.device_pb_plus.Device`

        server (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~ubii.proto.v1.servers.server_pb_plus.Server`

        session (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~ubii.proto.v1.sessions.session_pb_plus.Session`

        session_list (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~ubii.proto.v1.sessions.session_pb_plus.SessionList`

        processing_module (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~ubii.proto.v1.processing.processingModule_pb_plus.ProcessingModule`

        processing_module_list (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~ubii.proto.v1.processing.processingModule_pb_plus.ProcessingModuleList`

        string_list (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~ubii.proto.v1.dataStructure.lists_pb_plus.StringList`

        topic_mux (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~ubii.proto.v1.devices.topicMux_pb_plus.TopicMux`

        topic_mux_list (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~ubii.proto.v1.devices.topicMux_pb_plus.TopicMuxList`

        topic_demux (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~ubii.proto.v1.devices.topicDemux_pb_plus.TopicDemux`

        topic_demux_list (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~ubii.proto.v1.devices.topicDemux_pb_plus.TopicDemuxList`

        client_list (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~ubii.proto.v1.clients.client_pb_plus.ClientList`

        device_list (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~ubii.proto.v1.devices.device_pb_plus.DeviceList`

        service (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~ubii.proto.v1.services.service_pb_plus.Service`

        service_list (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~ubii.proto.v1.services.service_pb_plus.ServiceList`

        lockstep_processing_reply (proto.fields.Field): :obj:`~proto.fields.Field` of
            type
            :obj:`~ubii.proto.v1.processing.lockstepProcessing_pb_plus.LockstepProcessingReply`

    """

    success = Field(
        Success,
        number=1,
        oneof='type',
    )
    error = Field(
        Error,
        number=2,
        oneof='type',
    )
    client = Field(
        Client,
        number=3,
        oneof='type',
    )
    device = Field(
        Device,
        number=4,
        oneof='type',
    )
    server = Field(
        Server,
        number=5,
        oneof='type',
    )
    session = Field(
        Session,
        number=6,
        oneof='type',
    )
    session_list = Field(
        SessionList,
        number=7,
        oneof='type',
    )
    processing_module = Field(
        ProcessingModule,
        number=8,
        oneof='type',
    )
    processing_module_list = Field(
        ProcessingModuleList,
        number=9,
        oneof='type',
    )
    string_list = Field(
        StringList,
        number=10,
        oneof='type',
    )
    topic_mux = Field(
        TopicMux,
        number=11,
        oneof='type',
    )
    topic_mux_list = Field(
        TopicMuxList,
        number=12,
        oneof='type',
    )
    topic_demux = Field(
        TopicDemux,
        number=13,
        oneof='type',
    )
    topic_demux_list = Field(
        TopicDemuxList,
        number=14,
        oneof='type',
    )
    client_list = Field(
        ClientList,
        number=15,
        oneof='type',
    )
    device_list = Field(
        DeviceList,
        number=16,
        oneof='type',
    )
    service = Field(
        Service,
        number=17,
        oneof='type',
    )
    service_list = Field(
        ServiceList,
        number=18,
        oneof='type',
    )
    lockstep_processing_reply = Field(
        LockstepProcessingReply,
        number=19,
        oneof='type',
    )




"""
@generated by codestare-proto-plus.  Do not edit manually!
"""
from proto import (
    DOUBLE,
    Field,
    module,
)

from proto.message import (
    Message,
)


__protobuf__ = module(
    package="ubii.proto.v1",
    manifest={
        "Matrix3x2",
    }
)


class Matrix3x2(Message):
    """Generated from ubii/proto/v1/dataStructure/matrix3x2.proto

    Attributes:
        m00 (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~proto.DOUBLE`

        m01 (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~proto.DOUBLE`

        m10 (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~proto.DOUBLE`

        m11 (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~proto.DOUBLE`

        m20 (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~proto.DOUBLE`

        m21 (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~proto.DOUBLE`

    """

    m00 = Field(
        DOUBLE,
        number=1,
    )
    m01 = Field(
        DOUBLE,
        number=2,
    )
    m10 = Field(
        DOUBLE,
        number=3,
    )
    m11 = Field(
        DOUBLE,
        number=4,
    )
    m20 = Field(
        DOUBLE,
        number=5,
    )
    m21 = Field(
        DOUBLE,
        number=6,
    )




"""
@generated by codestare-proto-plus.  Do not edit manually!
"""
from proto import (
    Field,
    INT32,
    RepeatedField,
    STRING,
    module,
)

from proto.message import (
    Message,
)

from ubii.proto.v1.topicData.topicDataRecord_pb_plus import (
    TopicDataRecord,
)


__protobuf__ = module(
    package="ubii.proto.v1",
    manifest={
        "LockstepProcessingRequest",
        "LockstepProcessingReply",
    }
)


class LockstepProcessingRequest(Message):
    """Generated from ubii/proto/v1/processing/lockstepProcessing.proto

    Attributes:
        processing_module_ids (proto.fields.RepeatedField):
            :obj:`~proto.fields.RepeatedField` of type :obj:`~proto.STRING`

        records (proto.fields.RepeatedField): :obj:`~proto.fields.RepeatedField` of
            type
            :obj:`~ubii.proto.v1.topicData.topicDataRecord_pb_plus.TopicDataRecord`

        delta_time_ms (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~proto.INT32`

    """

    processing_module_ids = RepeatedField(
        STRING,
        number=1,
    )
    records = RepeatedField(
        TopicDataRecord,
        number=2,
    )
    delta_time_ms = Field(
        INT32,
        number=3,
    )


class LockstepProcessingReply(Message):
    """Generated from ubii/proto/v1/processing/lockstepProcessing.proto

    Attributes:
        processing_module_ids (proto.fields.RepeatedField):
            :obj:`~proto.fields.RepeatedField` of type :obj:`~proto.STRING`

        records (proto.fields.RepeatedField): :obj:`~proto.fields.RepeatedField` of
            type
            :obj:`~ubii.proto.v1.topicData.topicDataRecord_pb_plus.TopicDataRecord`

    """

    processing_module_ids = RepeatedField(
        STRING,
        number=1,
    )
    records = RepeatedField(
        TopicDataRecord,
        number=2,
    )




"""
@generated by codestare-proto-plus.  Do not edit manually!
"""
from proto import (
    Field,
    RepeatedField,
    STRING,
    module,
)

from proto.message import (
    Message,
)

from ubii.proto.v1.devices.topicDemux_pb_plus import (
    TopicDemux,
)

from ubii.proto.v1.devices.topicMux_pb_plus import (
    TopicMux,
)


__protobuf__ = module(
    package="ubii.proto.v1",
    manifest={
        "TopicInputMapping",
        "TopicInputMappingList",
        "TopicOutputMapping",
        "TopicOutputMappingList",
        "IOMapping",
        "IOMappingList",
    }
)


class TopicInputMapping(Message):
    """Generated from ubii/proto/v1/sessions/ioMappings.proto

    Attributes:
        input_name (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~proto.STRING`

        topic (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~proto.STRING`

        topic_mux (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~ubii.proto.v1.devices.topicMux_pb_plus.TopicMux`

    """

    input_name = Field(
        STRING,
        number=1,
    )
    topic = Field(
        STRING,
        number=2,
        oneof='topic_source',
    )
    topic_mux = Field(
        TopicMux,
        number=3,
        oneof='topic_source',
    )


class TopicInputMappingList(Message):
    """Generated from ubii/proto/v1/sessions/ioMappings.proto

    Attributes:
        elements (proto.fields.RepeatedField): :obj:`~proto.fields.RepeatedField` of
            type :obj:`~.TopicInputMapping`

    """

    elements = RepeatedField(
        TopicInputMapping,
        number=1,
    )


class TopicOutputMapping(Message):
    """Generated from ubii/proto/v1/sessions/ioMappings.proto

    Attributes:
        output_name (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~proto.STRING`

        topic (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~proto.STRING`

        topic_demux (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~ubii.proto.v1.devices.topicDemux_pb_plus.TopicDemux`

    """

    output_name = Field(
        STRING,
        number=1,
    )
    topic = Field(
        STRING,
        number=2,
        oneof='topic_destination',
    )
    topic_demux = Field(
        TopicDemux,
        number=3,
        oneof='topic_destination',
    )


class TopicOutputMappingList(Message):
    """Generated from ubii/proto/v1/sessions/ioMappings.proto

    Attributes:
        elements (proto.fields.RepeatedField): :obj:`~proto.fields.RepeatedField` of
            type :obj:`~.TopicOutputMapping`

    """

    elements = RepeatedField(
        TopicOutputMapping,
        number=1,
    )


class IOMapping(Message):
    """Generated from ubii/proto/v1/sessions/ioMappings.proto

    Attributes:
        processing_module_id (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~proto.STRING`

        input_mappings (proto.fields.RepeatedField): :obj:`~proto.fields.RepeatedField`
            of type :obj:`~.TopicInputMapping`

        output_mappings (proto.fields.RepeatedField):
            :obj:`~proto.fields.RepeatedField` of type
            :obj:`~.TopicOutputMapping`

        processing_module_name (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~proto.STRING`

    """

    processing_module_id = Field(
        STRING,
        number=1,
    )
    input_mappings = RepeatedField(
        TopicInputMapping,
        number=2,
    )
    output_mappings = RepeatedField(
        TopicOutputMapping,
        number=3,
    )
    processing_module_name = Field(
        STRING,
        number=4,
    )


class IOMappingList(Message):
    """Generated from ubii/proto/v1/sessions/ioMappings.proto

    Attributes:
        elements (proto.fields.RepeatedField): :obj:`~proto.fields.RepeatedField` of
            type :obj:`~.IOMapping`

    """

    elements = RepeatedField(
        IOMapping,
        number=1,
    )




"""
@generated by codestare-proto-plus.  Do not edit manually!

Note:
    __init__.py files for packages are autogenerated, and import all messages from submodules to keep import names consistent with the directory structure of the ``.proto`` source files.
"""
from .lockstepProcessing_pb_plus import (
    LockstepProcessingReply,
    LockstepProcessingRequest,
)

from .processingModule_pb_plus import (
    ModuleIO,
    ProcessingMode,
    ProcessingModule,
    ProcessingModuleList,
)


__all__ = (
    "ProcessingMode",
    "ModuleIO",
    "ProcessingModule",
    "ProcessingModuleList",
    "LockstepProcessingRequest",
    "LockstepProcessingReply",
)


"""
@generated by codestare-proto-plus.  Do not edit manually!
"""
from proto import (
    Field,
    INT32,
    module,
)

from proto.message import (
    Message,
)

from ubii.proto.v1.dataStructure.buttonEventType_pb_plus import (
    ButtonEventType,
)


__protobuf__ = module(
    package="ubii.proto.v1",
    manifest={
        "MouseEvent",
    }
)


class MouseEvent(Message):
    """Generated from ubii/proto/v1/dataStructure/mouseEvent.proto

    Attributes:
        type (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~ubii.proto.v1.dataStructure.buttonEventType_pb_plus.ButtonEventType`

        button (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~proto.INT32`

    """

    type = Field(
        ButtonEventType,
        number=1,
    )
    button = Field(
        INT32,
        number=2,
    )




"""
@generated by codestare-proto-plus.  Do not edit manually!
"""
from proto import (
    Field,
    RepeatedField,
    STRING,
    module,
)

from proto.message import (
    Message,
)

from ubii.proto.v1.dataStructure.pose3d_pb_plus import (
    Pose3D,
)

from ubii.proto.v1.dataStructure.vector3_pb_plus import (
    Vector3,
)


__protobuf__ = module(
    package="ubii.proto.v1",
    manifest={
        "Object3D",
        "Object3DList",
    }
)


class Object3D(Message):
    """Generated from ubii/proto/v1/dataStructure/object3d.proto

    Attributes:
        id (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~proto.STRING`

        pose (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~ubii.proto.v1.dataStructure.pose3d_pb_plus.Pose3D`

        size (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~ubii.proto.v1.dataStructure.vector3_pb_plus.Vector3`

        user_data_json (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~proto.STRING`

    """

    id = Field(
        STRING,
        number=1,
    )
    pose = Field(
        Pose3D,
        number=2,
    )
    size = Field(
        Vector3,
        number=3,
    )
    user_data_json = Field(
        STRING,
        number=4,
    )


class Object3DList(Message):
    """Generated from ubii/proto/v1/dataStructure/object3d.proto

    Attributes:
        elements (proto.fields.RepeatedField): :obj:`~proto.fields.RepeatedField` of
            type :obj:`~.Object3D`

    """

    elements = RepeatedField(
        Object3D,
        number=1,
    )




"""
@generated by codestare-proto-plus.  Do not edit manually!
"""
from proto import (
    Field,
    INT32,
    INT64,
    module,
)

from proto.message import (
    Message,
)


__protobuf__ = module(
    package="ubii.proto.v1",
    manifest={
        "Timestamp",
    }
)


class Timestamp(Message):
    """Generated from ubii/proto/v1/topicData/timestamp.proto

    Attributes:
        seconds (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~proto.INT64`

        nanos (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~proto.INT32`

        millis (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~proto.INT64`

    """

    seconds = Field(
        INT64,
        number=1,
    )
    nanos = Field(
        INT32,
        number=2,
    )
    millis = Field(
        INT64,
        number=3,
    )




"""
@generated by codestare-proto-plus.  Do not edit manually!
"""
from proto import (
    Enum,
    module,
)


__protobuf__ = module(
    package="ubii.proto.v1",
    manifest={
        "ButtonEventType",
    }
)


class ButtonEventType(Enum):
    UP = 0
    DOWN = 1



"""
Utility features to format protobuf messages and implement protobuf wrapper classes with custom behaviour.

Attributes:
    __imported_types__ (dict): module level dictionary caching imported types
"""

import collections
import functools
import importlib
import json
import logging
import re
from abc import ABCMeta
from typing import Dict

import google.protobuf.json_format
import google.protobuf.message
import proto.message

log = logging.getLogger(__name__)
__imported_types__ = {}

import_name = collections.namedtuple('import_name', ['package', 'type'])


def get_import_name(message_type: str):
    """
    The .proto files declare a package name 'ubii', but this is not reflected in the python package.
    Instead the package is named ubii.proto, to not clash with different packages in the ubii namespace.
    The python files are generated from updated proto files, to produce the correct directory structure.
    The directory structure determines python package names see
    https://developers.google.com/protocol-buffers/docs/reference/python-generated#package

    Args:
        message_type (str): string describing the data type

    """

    if not message_type.startswith('ubii.'):
        raise ValueError(f"{message_type} does not seem to be a special ubi-interact message type.")

    from . import __proto_module__
    package, typ = message_type.replace(
        'ubii', __proto_module__
    ).rsplit('.', maxsplit=1)

    return import_name(package=package, type=typ)


def import_type(message_type: str, reimport=False):
    """
    See the documentation of the ubii-msg-compiler at https://github.com/saggitar/ubii-msg-compiler
    for more information about updating the .proto files to generate a different package structure.

    Args:
        message_type (str): string describing the data type
        reimport (bool): If true force reimport of type

    """

    def _import(name: str) -> proto.Message:
        package, type_ = get_import_name(name)
        package = importlib.import_module(package)
        type_ = getattr(package, type_, None)

        if type_ is None:
            raise ValueError(f"{name} could not be imported.")
        return type_

    if message_type not in __imported_types__ or reimport:
        __imported_types__[message_type] = _import(message_type)

    return __imported_types__[message_type]


def clean_json(message: str):
    """
    Format json strings (like representations of proto messages) in a nice way.
    Close to ``re.sub(r'{\s+', '{', message.strip())``

    Returns:
        str: cleaned string
    """
    cleaned = message.strip()
    formatted = re.sub(r'{\s+', '{', cleaned)
    formatted = re.sub(r'\s', '', formatted)
    return formatted


def patch_wrapper_class_repr():
    def patch(__repr__):
        @functools.wraps(__repr__)
        def wrapped(self):
            type_ = self.__class__
            pb_type_descr = type_.pb().DESCRIPTOR
            vars_ = {name: getattr(self, name) for name in pb_type_descr.fields_by_name}

            def short_repr(value, max_len=50):
                orig = clean_json(str(value))
                if len(orig) < max_len:
                    return orig
                else:
                    return '...'

            formatted = {name: short_repr(value) for name, value in vars_.items() if value}
            return f"{type_.__name__}({formatted})"

            # return f"{name}({clean_json(old_repr)})"

        return wrapped

    proto.Message.__repr__ = patch(proto.Message.__repr__)


class ProtoEncoder(json.JSONEncoder):
    """
    Custom encoder to convert Protobuf Messages and Proto-Plus Messages to valid json.
    """

    #: dictionary of default arguments passed to the :meth:`proto.message.Message.to_dict` or
    #: :func:`google.protobuf.json_format.MessageToDict` calls which are used to convert the message object.
    format_options: Dict[str, bool] = {
        "use_integers_for_enums": True,
        "including_default_value_fields": True,
        "preserving_proto_field_name": False,
    }

    def default(self, o):
        """
        returns a serializable object for ``o``, or calls the base implementation (to raise a ``TypeError``).

        Args:
            o (object): (hopefully) serializable object, e.g. Protobuf message

        Returns:
            dict: Dictionary representation of message using :meth:`proto.message.Message.to_dict` or
            :func:`google.protobuf.json_format.MessageToDict`

        Raises:
            TypeError: If object is not serializable as Protobuf Message of by :meth:`json.JSONEncoder.default`

        """
        if isinstance(o, proto.Message):
            return type(o).to_dict(o, **self.format_options)

        if isinstance(o, google.protobuf.message.Message):
            return google.protobuf.json_format.MessageToDict(o, **self.format_options)

        return json.JSONEncoder.default(self, o)


class ProtoMeta(ABCMeta, proto.message.MessageMeta):
    """
    This metaclass is used to inherit from :doc:`plus:index` wrapper classes

    Example:
        Make sure you import and set :mod:`ubii.proto.__protobuf__` in every module
        which defines new :class:`~proto.message.Message` types::

            import ubii.proto as ub
            __protobuf__ = ub.__protobuf__

            class Fancy(ub.Component, metaclass=ub.ProtoMeta):
                @property
                def some_thing_amazing(self)
                    return "Wow"

    """

    def __new__(mcs, name, bases, attrs):
        message_bases = [b for b in bases if isinstance(b, proto.message.MessageMeta)]
        if len(message_bases) != 1:
            raise NotImplemented(f"Can't subclass with {len(message_bases)} Message parent classes")

        parent: proto.message.MessageMeta = message_bases[0]
        cls = super().__new__(mcs, name, bases, {**attrs, **parent.meta.fields})
        cls.meta._pb = parent.pb()
        return cls


__all__ = (
    "get_import_name",
    "import_type",
    "ProtoMeta",
    "ProtoEncoder",
    "patch_wrapper_class_repr"
)



"""
@generated by codestare-proto-plus.  Do not edit manually!
"""
from proto import (
    FLOAT,
    Field,
    module,
)

from proto.message import (
    Message,
)

from ubii.proto.v1.dataStructure.vector2_pb_plus import (
    Vector2,
)


__protobuf__ = module(
    package="ubii.proto.v1",
    manifest={
        "Pose2D",
    }
)


class Pose2D(Message):
    """Generated from ubii/proto/v1/dataStructure/pose2d.proto

    Attributes:
        position (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~ubii.proto.v1.dataStructure.vector2_pb_plus.Vector2`

        direction (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~ubii.proto.v1.dataStructure.vector2_pb_plus.Vector2`

        angle (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~proto.FLOAT`

    """

    position = Field(
        Vector2,
        number=1,
    )
    direction = Field(
        Vector2,
        number=2,
        oneof='orientation',
    )
    angle = Field(
        FLOAT,
        number=3,
        oneof='orientation',
    )




"""
@generated by codestare-proto-plus.  Do not edit manually!
"""
from proto import (
    Field,
    module,
)

from proto.message import (
    Message,
)

from ubii.proto.v1.dataStructure.quaternion_pb_plus import (
    Quaternion,
)

from ubii.proto.v1.dataStructure.vector3_pb_plus import (
    Vector3,
)


__protobuf__ = module(
    package="ubii.proto.v1",
    manifest={
        "Pose3D",
    }
)


class Pose3D(Message):
    """Generated from ubii/proto/v1/dataStructure/pose3d.proto

    Attributes:
        position (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~ubii.proto.v1.dataStructure.vector3_pb_plus.Vector3`

        quaternion (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~ubii.proto.v1.dataStructure.quaternion_pb_plus.Quaternion`

        euler (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~ubii.proto.v1.dataStructure.vector3_pb_plus.Vector3`

    """

    position = Field(
        Vector3,
        number=1,
    )
    quaternion = Field(
        Quaternion,
        number=2,
        oneof='orientation',
    )
    euler = Field(
        Vector3,
        number=3,
        oneof='orientation',
    )




"""
@generated by codestare-proto-plus.  Do not edit manually!
"""
from proto import (
    BOOL,
    DOUBLE,
    FLOAT,
    INT32,
    RepeatedField,
    STRING,
    module,
)

from proto.message import (
    Message,
)


__protobuf__ = module(
    package="ubii.proto.v1",
    manifest={
        "StringList",
        "DoubleList",
        "FloatList",
        "BoolList",
        "Int32List",
    }
)


class StringList(Message):
    """Generated from ubii/proto/v1/dataStructure/lists.proto

    Attributes:
        elements (proto.fields.RepeatedField): :obj:`~proto.fields.RepeatedField` of
            type :obj:`~proto.STRING`

    """

    elements = RepeatedField(
        STRING,
        number=1,
    )


class DoubleList(Message):
    """Generated from ubii/proto/v1/dataStructure/lists.proto

    Attributes:
        elements (proto.fields.RepeatedField): :obj:`~proto.fields.RepeatedField` of
            type :obj:`~proto.DOUBLE`

    """

    elements = RepeatedField(
        DOUBLE,
        number=1,
    )


class FloatList(Message):
    """Generated from ubii/proto/v1/dataStructure/lists.proto

    Attributes:
        elements (proto.fields.RepeatedField): :obj:`~proto.fields.RepeatedField` of
            type :obj:`~proto.FLOAT`

    """

    elements = RepeatedField(
        FLOAT,
        number=1,
    )


class BoolList(Message):
    """Generated from ubii/proto/v1/dataStructure/lists.proto

    Attributes:
        elements (proto.fields.RepeatedField): :obj:`~proto.fields.RepeatedField` of
            type :obj:`~proto.BOOL`

    """

    elements = RepeatedField(
        BOOL,
        number=1,
    )


class Int32List(Message):
    """Generated from ubii/proto/v1/dataStructure/lists.proto

    Attributes:
        elements (proto.fields.RepeatedField): :obj:`~proto.fields.RepeatedField` of
            type :obj:`~proto.INT32`

    """

    elements = RepeatedField(
        INT32,
        number=1,
    )




"""
@generated by codestare-proto-plus.  Do not edit manually!
"""
from proto import (
    Field,
    RepeatedField,
    STRING,
    module,
)

from proto.message import (
    Message,
)


__protobuf__ = module(
    package="ubii.proto.v1",
    manifest={
        "TopicSubscription",
    }
)


class TopicSubscription(Message):
    """Generated from ubii/proto/v1/services/request/topicSubscription.proto

    Attributes:
        client_id (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~proto.STRING`

        subscribe_topics (proto.fields.RepeatedField):
            :obj:`~proto.fields.RepeatedField` of type :obj:`~proto.STRING`

        unsubscribe_topics (proto.fields.RepeatedField):
            :obj:`~proto.fields.RepeatedField` of type :obj:`~proto.STRING`

        subscribe_topic_regexp (proto.fields.RepeatedField):
            :obj:`~proto.fields.RepeatedField` of type :obj:`~proto.STRING`

        unsubscribe_topic_regexp (proto.fields.RepeatedField):
            :obj:`~proto.fields.RepeatedField` of type :obj:`~proto.STRING`

    """

    client_id = Field(
        STRING,
        number=1,
    )
    subscribe_topics = RepeatedField(
        STRING,
        number=2,
    )
    unsubscribe_topics = RepeatedField(
        STRING,
        number=3,
    )
    subscribe_topic_regexp = RepeatedField(
        STRING,
        number=4,
    )
    unsubscribe_topic_regexp = RepeatedField(
        STRING,
        number=5,
    )




"""
@generated by codestare-proto-plus.  Do not edit manually!

Note:
    __init__.py files for packages are autogenerated, and import all messages from submodules to keep import names consistent with the directory structure of the ``.proto`` source files.
"""
from .serviceReply_pb_plus import (
    ServiceReply,
)

from .serviceRequest_pb_plus import (
    ServiceRequest,
)

from .service_pb_plus import (
    Service,
    ServiceList,
)


__all__ = (
    "Service",
    "ServiceList",
    "ServiceReply",
    "ServiceRequest",
)


"""
@generated by codestare-proto-plus.  Do not edit manually!
"""
from proto import (
    Field,
    RepeatedField,
    STRING,
    module,
)

from proto.message import (
    Message,
)


__protobuf__ = module(
    package="ubii.proto.v1",
    manifest={
        "TopicDemux",
        "TopicDemuxList",
    }
)


class TopicDemux(Message):
    """Generated from ubii/proto/v1/devices/topicDemux.proto

    Attributes:
        id (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~proto.STRING`

        name (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~proto.STRING`

        data_type (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~proto.STRING`

        output_topic_format (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~proto.STRING`

    """

    id = Field(
        STRING,
        number=1,
    )
    name = Field(
        STRING,
        number=2,
    )
    data_type = Field(
        STRING,
        number=3,
    )
    output_topic_format = Field(
        STRING,
        number=4,
    )


class TopicDemuxList(Message):
    """Generated from ubii/proto/v1/devices/topicDemux.proto

    Attributes:
        elements (proto.fields.RepeatedField): :obj:`~proto.fields.RepeatedField` of
            type :obj:`~.TopicDemux`

    """

    elements = RepeatedField(
        TopicDemux,
        number=1,
    )




"""
@generated by codestare-proto-plus.  Do not edit manually!
"""
from proto import (
    DOUBLE,
    Field,
    RepeatedField,
    module,
)

from proto.message import (
    Message,
)


__protobuf__ = module(
    package="ubii.proto.v1",
    manifest={
        "Vector8",
        "Vector8List",
    }
)


class Vector8(Message):
    """Generated from ubii/proto/v1/dataStructure/vector8.proto

    Attributes:
        v0 (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~proto.DOUBLE`

        v1 (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~proto.DOUBLE`

        v2 (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~proto.DOUBLE`

        v3 (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~proto.DOUBLE`

        v4 (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~proto.DOUBLE`

        v5 (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~proto.DOUBLE`

        v6 (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~proto.DOUBLE`

        v7 (proto.fields.Field): :obj:`~proto.fields.Field` of type
            :obj:`~proto.DOUBLE`

    """

    v0 = Field(
        DOUBLE,
        number=1,
    )
    v1 = Field(
        DOUBLE,
        number=2,
    )
    v2 = Field(
        DOUBLE,
        number=3,
    )
    v3 = Field(
        DOUBLE,
        number=4,
    )
    v4 = Field(
        DOUBLE,
        number=5,
    )
    v5 = Field(
        DOUBLE,
        number=6,
    )
    v6 = Field(
        DOUBLE,
        number=7,
    )
    v7 = Field(
        DOUBLE,
        number=8,
    )


class Vector8List(Message):
    """Generated from ubii/proto/v1/dataStructure/vector8.proto

    Attributes:
        elements (proto.fields.RepeatedField): :obj:`~proto.fields.RepeatedField` of
            type :obj:`~.Vector8`

    """

    elements = RepeatedField(
        Vector8,
        number=1,
    )

